# hist.tcl--- histogrammer control frame

#    new version 11-08-98  VP


set debug 0
set started 0

set frame HI
set frametitle "Hardware Histogrammer"
set icontitle "Histogramer"

#    size of detector overlay panel

set ovlyw 350
set ovlyh 100

#    current hist server resource
set resource ""
#    list of available hist server resources
set resources {}
#    number of available hist server resources
set numresources 0

set sorttype numeric

#    optional configuration file which contains map of VXI crates to hist servers
set HistDefn [file join $env(MIDASAPPLBASE) config VXI HistDefn]

#    supported detector types
set detectormenu [list Tapered Clover Cluster BgoBall Diamant Euclides Nwall Trigger TriggerMK2 Saphir CsI Icare QDC6414V ADC6414V QDC3214V ADC3214V TDC3214V ESS HRGe8x ECC GOCCE TDR32V FVI VRE ROCO Group]
set detector ""

#    directory selection patterns
set patternmenu {"" *q0 *q1 *q2 *q3 *20MeV* *4MeV* *Energy* *TAC* *Time* *Timing* *Adc* Ge* BGO* Clover* Cluster* Diamant* Euclides* Nwall* Saphir* CsI* Icare* QDC6414V* ADC6414V* QDC3214V* ADC3214V* TDC3214V* ESS* HRGe8x* ECC* GOCCE* TDR32V*}
set pattern ""

set lowchannel 0
set highchannel 0
set qualifier 1
set maxchannel 255
set maxgroup 255
set lowitem 0
set highitem 0
set maxitem 63

set dirlist {}
set spectrumlist {}


proc list-take {as is} {
   set ll {}
   foreach e $is {lappend ll [lindex $as $e]}
   return $ll
}

proc prefix-list {l p} {
   set ll {}
   foreach f $l {lappend ll $p$f}
   return $ll
}

#    The section of code which follows handles spectrum server access to
#    create, delete and clear spectra
#    These actions have to be directed to the correct server when there
#    are more than one server in the experiment

set vxicontrolstarted 0

#    procedure to return VXI crate holding a module

proc VXImodule2crate {module} {
  global vxicontrolstarted

#    obtain crate holding a specific module
#    this uses the task  "vxicontrol"

  if {$vxicontrolstarted == 0} then {
     set started [inform vxicontrol inquire started]
     if {$started == 0} then {inform vxicontrol make-frame}
     set vxicontrolstarted 1
  }

  set crate [send vxicontrol VXImodule2crate $module]
  return $crate
}

if {[file exists $HistDefn] == 1} then {source $HistDefn}

proc execute-spectrum {n action} {
  global env Hist resource numresources
  global debug

#insert-debug "$action $n"

  if {$numresources <= 1 || ![info exists Hist]} then {set server $resource} else {
#
#    more than 1 server
#    must send action to the appropiate server
#
     set vximodule  [first [split $n .]]
     set moduletype [first [split $vximodule 0123456789]]
#
#    for all detectors connected via the FVI interface we need the location
#    of the FVI interface
#    this needs to be improved!!
#    wait until we sort out multiple FVI interfaces!!!
#
#    Note Nwall is not via the FVI now but using it will get the correct hist server since 
#    the FVI for Euroball is in the ancillary detector crate
#
#    There are multiple FVI interfaces in Euroball but again assume that both will be in the 
#    ancillary detector VXI crate

     if {$moduletype == "Diamant"} then {set vximodule "FVI1"}
     if {$moduletype == "Nwall"}   then {set vximodule "FVI1"}
     if {$moduletype == "Euclides"}   then {set vximodule "FVI1"}
#
     set vxicrate [VXImodule2crate $vximodule]

#    name of global Hardware Histogrammer resource
     if {[info exists env(HwreHistServer)] == 0} then {set r "hist"}  \
        else {set r $env(HwreHistServer)}

     if {$resource == $r} then {
        if {[info exists Hist($vxicrate)]} then {set server $Hist($vxicrate)} \
           else {set server $resource}
     } else {
        if {$resource == "rmhist"} then {set server $vxicrate} else {set server $resource}
     }
  }

#insert-debug "$server"

  switch $action {
     create {
        set range [range-of $n]
        set specname $n
        if {$debug} then {insert-debug "create spectrum :range $range :name '$specname' "}
        set z [catch {eg "(spectrum :path '/$server' :range $range :name '$specname' !new)" } m]
#    Allow "Spectrum already exists" so that further spectra can be created
        if {$z!=0 && $m==0x30004} then {set z 0}
     }
     delete {
        if {$debug} then {insert-debug "delete spectrum  :name '$n' "}
        set z [catch {eg "(spectrum :path '/$server' :name '$n' !delete)"} m]
# Allow "Pathname Invalid" so that further spectra can be deleted
        if {$z!=0 && $m==0x30004} then {set z 0}
     }
     clear {
        if {$debug} then {insert-debug "clear spectrum  :name '$n' "}
        set z [catch {eg "(spectrum :path '/$server' :name '$n' !zero)"} m]
     }
  }
  if {$z!=0} then {
    display-eg-error-report self $m "Attempting to " $action "spectrum" $n
    clear-footer
    error $m
  }
}

proc create-spectrum n {execute-spectrum $n create}

proc delete-spectrum n {execute-spectrum $n delete}

proc zero-spectrum n {execute-spectrum $n clear}

proc read-histogrammer {} {
    global resource pattern dirlist
    if {"$pattern" != ""} then {set path "/$resource/$pattern"} else {set path "/$resource"}
    set z [catch {eg "(spectrum :path '$path' ?names)"} m]
    if {$z!=0} then {
       display-eg-error-report self $m "Attempting to read histogrammer" $path
       clear-footer
       error $m
    }
    set dirlist $m
    sort-dirlist
}

proc compare-numeric {a b} {
   set pat "(\[^0-9\]+)(\[0-9\]+)(.*)(\.*)"
   set za [regexp $pat $a a0 a1 a2 a3 a4]
   set zb [regexp $pat $b b0 b1 b2 b3 b4]
   if {$za==0 || $zb==0} then {return [string compare $a $b]}
   if {"$a"!="$a0" || "$b"!="$b0"} then {return [string compare $a $b]}
   set z [string compare $a1 $b1]
   if {$z!=0} then {return $z}
   set z [expr int($a2.0-$b2.0)]
   if {$z!=0} then {return $z}
   return [string compare $a $b]
}

proc sort-dirlist {} {
   global sorttype dirlist
   set ll {}
   foreach f $dirlist {lappend ll [second $f]\\[first $f]}
   if {"$sorttype"=="numeric"} then \
      {set ll [lsort -command compare-numeric $ll]} \
   elseif {"$sorttype"=="alphabetic"} then \
      {set ll [lsort -ascii $ll]}
   set dirlist {}
   foreach f $ll {
      set s [split $f "\\"]
      lappend dirlist [first $s]
   }
}

proc paint-histlist {} {
   global env dirlist spectrumlist
   :modify LIST :append F :strings $dirlist :see 0
   set spectrumlist {}
}

proc paint-spectrumlist {} {
  global env dirlist spectrumlist
  foreach s $spectrumlist {
    set i [lsearch $dirlist $s]
    if {$i >= 0} then {:modify LIST :select $i}
  }
}

proc click-zero {} {
   global resource
   set c 0
   foreach spectrum [get-spectrum-list] {
      set-footer "zeroing /$resource/$spectrum..."
      zero-spectrum $spectrum
      incr c
   }
   set-footer "total of $c spectra successfully zeroed"
}

proc click-create {} {
   global resource

   Make_Xlate  $resource

   set c 0
   foreach spectrum [get-spectrum-list] {
      set-footer "creating /$resource/$spectrum..."
      create-spectrum $spectrum
      incr c
   }
   click-redisplay
   set-footer "total of $c spectra successfully created"
}

proc click-delete {} {
   global resource
   set c 0
   foreach spectrum [get-spectrum-list] {
      set-footer "deleting /$resource/$spectrum..."
      delete-spectrum $spectrum
      incr c
   }
   click-redisplay
   set-footer "total of $c spectra successfully deleted"
}

proc click-show {} {
   global resource
   set list [get-spectrum-list]
#   puts stderr  "show [prefix-list $list /$resource/]"
   inform sd  "show [prefix-list $list /$resource/]"
}

proc click-addresses {} {
        global resource
        set list [prefix-list [get-spectrum-list] /$resource/]
        if {$list=={}} then {
                set-footer "you haven't selected any spectra!!!";
                return
        }
        clear-footer
        foreach sp $list {
                set z [catch {eg "(spectrum  :name '$sp' ?addr)"} m]
                if {$z!=0} then {
                        display-eg-error-report self $m "Attempting to find address of spectrum" $sp
                        clear-footer
                        error $m
                }
                insert-log [transparent [format "%s:\t0x%x" [lindex [split $sp /] 2] $m]]
        }
}

proc click-histogrammer d {
   global resource
   set resource $d
   :modify HISTOGRAMMER :value $resource
   click-redisplay
}

proc make-resource-list {} {
  global env resource resources numresources

   set z [catch {eg "(experiment ?resources)"} m]
   if {$z==0} then {set rs $m} else {set rs ""}

#    scan through resources and count instances of all hist spectrum servers
   if {[info exists env(HwreHistServer)] == 0} then {set role hist}  \
      else {set role $env(HwreHistServer)}
   set count 0
   set rolecount(hist) 0
   set rolecount(rmhist) 0
   foreach r $rs {
      set type [second [fourth $r]]
      set role [second [fifth $r]]
      if {$type == "spectrum" && $role == "hist"} then {incr count; incr rolecount(hist)}
      if {$type == "spectrum" && $role == "rmhist"} then {incr count; incr rolecount(rmhist)}
   }

   if {[info exists env(HwreHistServer)] == 0} then {set resource hist}  \
      else {set resource $env(HwreHistServer)}
   set resources {}
   set numresources 0
   if {$rolecount(hist) > 0}   then {lappend resources hist; incr numresources}
   if {$rolecount(rmhist) > 0} then {lappend resources rmhist; incr numresources}

   if {$count > 1 || $role != $resource} then {
#    next scan through resources and add instances of all hist spectrum servers to the list
      foreach r $rs {
         set type [second [fourth $r]]
         set role [second [fifth $r]]
         set name [second [first $r]]
         if {$type == "spectrum" && $role == "hist" && $rolecount(hist) > 1}   then {lappend resources $name; incr numresources}
         if {$type == "spectrum" && $role == "rmhist" && $rolecount(rmhist) > 1} then {lappend resources $name; incr numresources}
      }
   } else {
      if {[info exists env(HwreHistServer)] == 0} then {set resources hist}  \
         else {set resources $env(HwreHistServer)}
      set numresources 1
   }
   if {[info exists env(HwreHistServer)] == 0} then {set resource [first $resources]}  \
      else {set resource $env(HwreHistServer)}
}

proc click-list {c s} {
  global spectrumlist
  set spectrumlist $s
  paint-spectrumlist
}

proc click-pattern {p} {
   global  pattern spectrumlist
   :modify PATTERN :value $p
   set pattern $p
   set spectrumlist {}
   click-redisplay
}

proc click-detector d {
   global detector spectrumlist
   set detector $d
   :modify DETECTOR :value $detector
   read_detector_overlay
   paint_detector
   set spectrumlist {}
   click-redisplay
}

proc read_detector_overlay {} {
   global detector
#
#    we attempt to read in a code overlay for this detector type
#    file name has the format  hist_detector.tcl
#
  if {[loadsource [string tolower hist_$detector]] == 0} then {
     midas-warning "No code overlay found for detector type $detector"
  }
}

proc paint_detector {} {
   global ovlyw ovlyh
    catch {:modify OVERLAY :delete T}
    :panel OVERLAY :layout L :padx 5 :pady 0 :w $ovlyw :h $ovlyh :before SPECTRA
    [:path OVERLAY] configure -relief raised -bd 1
    make-overlaypanel
}

proc click-low v {
   global lowchannel highchannel maxchannel
   set lowchannel $v
   if {$lowchannel<0} then {set lowchannel 0}
   if {$lowchannel>$maxchannel} then {set lowchannel $maxchannel}
   if {$highchannel<$lowchannel} then {set highchannel $lowchannel}
   repaint
}

proc click-high v {
   global lowchannel highchannel maxchannel
   set highchannel $v
   if {$highchannel<0} then {set highchannel 0}
   if {$highchannel>$maxchannel} then {set highchannel $maxchannel}
   if {$lowchannel>$highchannel} then {set lowchannel $highchannel}
   repaint
}

proc click-qualifier v {
   global qualifier spectrumlist
   set qualifier $v
   if {$spectrumlist!={}} then {paint-histlist; repaint}
}

proc click-deselect-all {} {
   global spectrumlist
   set spectrumlist {}
   :modify LIST :select -1
}

proc click-select-all {} {
   global env dirlist spectrumlist
   read-histogrammer
   paint-histlist
   set i 0
   :modify LIST :show F
   foreach s $dirlist {:modify LIST :select $i; incr i}
   :modify LIST :show T
   set spectrumlist $dirlist
   repaint
}

proc click-adc {v adc} {
# click-adc  select an ADC and deselect the list
   global $adc spectrumlist
   set $adc $v
   if {$spectrumlist != {}} then {paint-histlist; repaint}
}

proc repaint {} {
  global resource lowchannel highchannel qualifier

     :modify HISTOGRAMMER :value $resource

     catch {
       :modify DET-LOW :value $lowchannel
       :modify DET-HIGH :value $highchannel
       :modify DET-QUAL :value $qualifier
     }

     repaint_overlay
}

proc repaint_overlay {} {}

proc click-redisplay {} {
   read-histogrammer
   paint-histlist
   repaint
}

proc click arg {
   global debug hsdir
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit $f; return}
   switch -glob $w {
      ZERO              {click-zero}
      CREATE            {click-create}
      DELETE            {click-delete}
      SHOW              {click-show}
      SAVE              {inform speccopy start hist}
      ADDRESS           {click-addresses}
      HISTOGRAMMER -
      HISTOGRAMMER-MENU {click-histogrammer $v}
      DETECTOR -
      DETECTOR-MENU     {click-detector $v}
      PATTERN -
      PATTERN-MENU      {click-pattern $v}
      LIST              {click-list $c $v}
      DESELECT-ALL      {click-deselect-all}
      SELECT-ALL        {click-select-all}
      REDISPLAY         {click-redisplay}
      DET-LOW           {click-low $v}
      DET-HIGH          {click-high $v}
      DET-QUAL          {click-qualifier $v}
      DET-*             {click-overlay $w $v}
      default    {puts stdout $arg}
   }
}

#    the following procedure should be defined in the detector overlay script
proc click-overlay {w} {midas-warning "no detector handler defined"}

proc start args {
   global env started detectormenu
   if {$started != 0} then {open-frame} else {
      make-resource-list
      make-frame
      click-detector [first $detectormenu]
      set started 1
      read-histogrammer
      paint-histlist
      repaint
   }
}

proc click-quit {f} {
   global env started
   if {$f=="SEL"} then {destroy-frame SEL; return}
   destroy-frame SEL
   close-frame
}

proc finish args {destroy-frame SEL; destroy-frame; set started 0}



proc make-frame {} {
  global env resources detectormenu patternmenu
  global ovlyw ovlyh

   :frame [self frame-name] :label "[self frame-title]" :show-footer T  :resize F \
        :icon-image "@$env(TCLHOME)/bitmaps/spectrum" \
        :icon-name "[self icon-title]"

   :panel P1 :layout T :padx 10 :pady 10
      :layout H; :col-gap 5

      :menubutton DETECTOR-MENU     :label "Detector"      :menu $detectormenu
      :text DETECTOR     :label "" :w 12
      :hskip 100
      :menubutton HISTOGRAMMER-MENU :label "Histogrammer:" :menu $resources
      :text HISTOGRAMMER :label "" :w 12

    :panel SPECTRA :layout R :padx 5 :pady 0
      :list LIST :cols 28 :rows 18 :exclusive F

    :panel P4 :layout B :padx 10 :pady 10 :after FOOTER
      :layout H; :row-gap 10
      :col-gap 30
      :button CREATE  :label Create
      :button DELETE  :label Delete
      :button ZERO    :label Zero
      :button SAVE    :label Save...
      :hskip 50
      :button BROWSER :label "Spectrum Viewer..." :command "inform sb histstart"
      :button HELP  :label Help

      :next-row T
      :col-gap 10
      :button SHOW          :label Show
      :button SELECT-ALL    :label "Select all"
      :button DESELECT-ALL  :label "Deselect all"

      :menubutton PATTERN-MENU  :label "Selection"  :menu $patternmenu
      :col-gap 0
      :text PATTERN :label "" :w 20

      :hskip 50
      :button REDISPLAY     :label Redisplay

    :panel OVERLAY :layout L :padx 5 :pady 0 :w $ovlyw :h $ovlyw :before SPECTRA
     [:path OVERLAY] configure -relief raised -bd 1

    :show T
}


