
set LYRTECH_ServerName_Default LyrTech

set lyrtech_debug 0

set LyrTechServers {}

proc Custom_Start {} {
    global Options LYRTECH_ServerName_Default

   if {![info exists Options(LYRTECH_ServerName)]} {set Options(LYRTECH_ServerName) $LYRTECH_ServerName_Default}

   update_Options

   LyrTech_Servers
}

proc LyrTech_Servers {} {
   global Options
   global DataAcquisitionControlServerHost DataAcquisitionControlServerPort
   global DataAcquisitionControlClient__Debug
   global lyrtech_debug LyrTechServers

   package require DataAcquisitionControlClient

   set ServerName $Options(LYRTECH_ServerName)

   set LyrTechServers {}
   foreach HostPort [inform ex obtain_SOAPServers_byName $ServerName] {
     lappend LyrTechServers $HostPort
     set DataAcquisitionControlServerHost [first [split $HostPort :]]
     set DataAcquisitionControlServerPort [second [split $HostPort :]]
#     DataAcquisitionControlClient_Configure
     if {$lyrtech_debug} {
       insert-debug "Creating SOAP clients for $HostPort"
       set DataAcquisitionControlClient__Debug 1
     }
     DataAcquisitionControlClient_Setup "$HostPort"
  }
}

proc NoLyrTech_Servers {} {
   global LyrTechServers
   foreach HostPort $LyrTechServers {
      DataAcquisitionControlClient_Destroy $HostPort
   }
   set LyrTechServers {}
}

proc LyrTech_Stop {} {
   global lyrtech_debug LYRTECHEnable LyrTechServers
   if {$lyrtech_debug} {
      if {$LYRTECHEnable != 0} {set not ""} {set not "not "}
      insert-debug "Will [set not]call LyrTech \"Stop\""
   }

   if {$LYRTECHEnable != 0} then {
     set ret -1
     foreach HostPort $LyrTechServers {
# Force server to have correct/up-to-date state
        set z [ catch { [set HostPort]_DataAcquisitionControlClient__GetState } response ]
        if {$z} {
           midas-warning "Cannot contact DataAcquisitionControlClient at $HostPort"
           continue
        }
#        set response [[set HostPort]_DataAcquisitionControlClient__GetState]
        if {$lyrtech_debug} {
          insert-debug "Calling [set HostPort]_DataAcquisitionControlClient__DoStop"
        }
        set ret [[set HostPort]_DataAcquisitionControlClient__DoStop]
        if {$lyrtech_debug} {
          insert-debug "DoStop returned $ret"
        }
     }
   } else {
     set ret -1
   }
   return $ret
}

proc LyrTech_Go {} {
   global lyrtech_debug LYRTECHEnable LyrTechServers
   if {$lyrtech_debug} {
      if {$LYRTECHEnable != 0} {set not ""} {set not "not "}
      insert-debug "Will [set not]call LyrTech \"Go\""
   }

   if {$LYRTECHEnable != 0} then {
     set ret -1
     foreach HostPort $LyrTechServers {
# Force server to have correct/up-to-date state
#        set response [[set HostPort]_DataAcquisitionControlClient__GetState]
        set z [ catch { [set HostPort]_DataAcquisitionControlClient__GetState } response ]
        if {$z} {
           midas-warning "Cannot contact DataAcquisitionControlClient at $HostPort"
           continue
        }
        set ret [[set HostPort]_DataAcquisitionControlClient__DoGo]
     }
   } else {
     set ret -1
   }
}

# opt is 0 (where needed), 1 (everything) or 2 (prompt)
proc LyrTech_Setup opt {
   global lyrtech_debug LYRTECHEnable LyrTechServers
   if {$lyrtech_debug} {
      insert-debug "Entered LyrTech_Setup - option $opt"
      if {$LYRTECHEnable == 0} {insert-debug "Will not call \"Setup\" routines"}
   }

   if {$LYRTECHEnable != 0} then {
    set ret -1
    set tmp $opt
    foreach HostPort $LyrTechServers {
      set opt $tmp
      if {$opt == 2} then {
            if {$lyrtech_debug != 0} {insert-debug "Will call LyrTech \"GetState\" ($HostPort)"}
            set z [ catch { [set HostPort]_DataAcquisitionControlClient__GetState } response ]
            if {$z} {
               midas-warning "Cannot contact DataAcquisitionControlClient at $HostPort"
               continue
            }
            set code [second $response]
#            set code [second [[set HostPort]_DataAcquisitionControlClient__GetState]]
            if {$code == 2} {set doneit "NOT"; set rec0 " (recommended)"; set rec1 ""} {set doneit "ALREADY"; set rec1 " (recommended)"; set rec0 ""}
            set r [midas-confirmation "The LyrTech control software at $HostPort has $doneit performed a setup.\n"\
               "Perform setup?$rec0"  "Skip to next step$rec1"]
            if {$r == 0} {set opt 1} {set opt -1}
      }
      if {$opt != -1} then {
# if opt is setup everything force to reset state - ready for setup
        if {$opt == 1} then {
           if {$lyrtech_debug != 0} {insert-debug "Will call LyrTech \"Reset\" ($HostPort)"}
           set z [ catch { [set HostPort]_DataAcquisitionControlClient__DoReset } response ]
           if {$z} {
               midas-warning "Cannot contact DataAcquisitionControlClient at $HostPort"
               continue
           }
#           set ret [[set HostPort]_DataAcquisitionControlClient__DoReset]
        }
        if {$lyrtech_debug != 0} {insert-debug "Will call LyrTech \"GetState\" ($HostPort)"}
        set z [ catch { [set HostPort]_DataAcquisitionControlClient__GetState } response ]
        if {$z} {
           midas-warning "Cannot contact DataAcquisitionControlClient at $HostPort"
           continue
        }
#        set response [[set HostPort]_DataAcquisitionControlClient__GetState]
#        set rc [first $response]
        set code [second $response]
        if {$lyrtech_debug != 0} {
           set state [third $response]
           set reason [fourth $response]
           insert-debug "state code is now $code \[$state $reason\]"
        }
# if resetting wait a bit (can be a few seconds)
        set wait 0
        set code -1
        while {$code == 7 && $wait < 15} {
           after 1000
           incr wait
           if {$lyrtech_debug != 0} {insert-debug "Will call LyrTech \"GetState\" ($HostPort)"}
           set z [ catch { [set HostPort]_DataAcquisitionControlClient__GetState } response ]
           if {$z} {
              midas-warning "Cannot contact DataAcquisitionControlClient at $HostPort"
              continue
           }
#           set response [[set HostPort]_DataAcquisitionControlClient__GetState]
#           set rc [first $response]
           set code [second $response]
           if {$lyrtech_debug != 0} {
              set state [third $response]
              set reason [fourth $response]
              insert-debug "state code is now $code \[$state $reason\]"
           }
        }
# only do the setup if we are in the reset state
        if {$code == 2} then {
           if {$lyrtech_debug != 0} {insert-debug "Will call LyrTech \"Setup\" ($HostPort)"}
           set ret [[set HostPort]_DataAcquisitionControlClient__DoSetUp]
        } else {
           if {$code == 7} {midas-warning "Reset has not completed\nGiving up waiting"}
           clear-footer
           set ret 0
        }
      } else {
        set ret -2
      }
     }
   } else {
      set ret -1
   }
}

proc LyrTech_Reset {force} {
   global lyrtech_debug LYRTECHEnable LyrTechServers
   if {$lyrtech_debug} {
      if {$LYRTECHEnable != 0} {set not ""} {set not "not "}
      if {$force != 0} {set not ""} {set not "not "}
      insert-debug "Will [set not]call LyrTech \"Reset\""
   }

   if {$LYRTECHEnable != 0 && $force != 0} then {
     foreach HostPort $LyrTechServers {
# Force server to have correct/up-to-date state
        set z [ catch { [set HostPort]_DataAcquisitionControlClient__GetState } response ]
        if {$z} {
           midas-warning "Cannot contact DataAcquisitionControlClient at $HostPort"
           continue
        }
#        set response [[set HostPort]_DataAcquisitionControlClient__GetState]
        set ret [[set HostPort]_DataAcquisitionControlClient__DoReset]
     }
   } else {
     if {$lyrtech_debug} {
       insert-debug "This \"Reset\" is currently a no-op"
     }
     set ret -1
   }
}

proc LyrTech_SetSync tim {
   global lyrtech_debug LYRTECHEnable LyrTechServers
   if {$lyrtech_debug} {
      if {$LYRTECHEnable != 0} {set not ""} {set not "not "}
      insert-debug "Will [set not]call LyrTech \"SetSync\" [format 0x%x $tim]"
   }

   if {$LYRTECHEnable != 0} then {
     foreach HostPort $LyrTechServers {
# Force server to have correct/up-to-date state
        set z [ catch { [set HostPort]_DataAcquisitionControlClient__GetState } response ]
        if {$z} {
            midas-warning "Cannot contact DataAcquisitionControlClient at $HostPort"
            continue
        }
#        set response [[set HostPort]_DataAcquisitionControlClient__GetState]
        set ret [[set HostPort]_DataAcquisitionControlClient__DoOption  ReSyncTime $tim]
     }
   } else {
     set ret -1
   }
}

proc LyrTech_ReadState {} {
   global lyrtech_debug LYRTECHEnable LyrTechServers
   if {$lyrtech_debug} {
      if {$LYRTECHEnable != 0} {set not ""} {set not "not "}
#set not ""
      insert-debug "Will [set not]call LyrTech \"ReadState\" "
   }

# if any system is going the returned state is going
# else if any system if neither stopped nor going returned state is unknown
# read all servers to update their own internal state
# return 0 - all stopped else non-stopped state from last server in list
   if {$LYRTECHEnable == 0} then {
      set ret -1; # (unknown)
   } else {
   set ret 0; # (stopped)
   foreach HostPort $LyrTechServers {
     set z [ catch { [set HostPort]_DataAcquisitionControlClient__GetState } response ]
     if {$z} {
         midas-warning "Cannot contact DataAcquisitionControlClient at $HostPort"
         continue
     }
#     set response [[set HostPort]_DataAcquisitionControlClient__GetState]
#     set rc [first $response]
     set code [second $response]
     if {$lyrtech_debug} {
     set state [third $response]
     set reason [fourth $response]
          insert-debug "[set HostPort]_DataAcquisitionControlClient__GetState returned $code (state $state, reason $reason)"
     }
#     set state [third $response]
#     set reason [fourth $response]
     if {$code != 0} {set ret $code;# break}
   }
   }

   return $ret
}

